import "moebius92.PartyFrame.DropDownMenu";
import "moebius92.PartyFrame.PartyFrameOptionsButton";
import "moebius92.PartyFrame.PartyFrameOptionsControl";
import "moebius92.PartyFrame.PartyFrameOptionsEffect";
import "moebius92.PartyFrame.PartyFrameOptionsField";
import "moebius92.PartyFrame.PartyFrameOptionsGroup";

PartyFrameOptions = class(Turbine.UI.Control);

function PartyFrameOptions:Constructor(window)
	Turbine.UI.Control.Constructor(self);

	self.window = window;
	self.width = 100;
	self.size = 20;

	self:LoadControls();
	self:LoadLogSaved();
	self:LoadControlsSaved();
	self:UpdateControls();
	self:DefaultControl(0, self.window.layout);

	self:SetBackColor(Turbine.UI.Color(1.0, 0.0, 0.0, 0.3));

	local t = { };
	local i = 1;
	for k, v in ipairs(layouts) do
		if v == self.window.layout then
			i = k;
		end
		table.insert(t, v.name);
	end
	self.layoutDD = DropDownMenu(self, i, t, function(self, index)
		self.parent.window.effectControl = { };
		self.parent.window:ClearMembers();
		for k in ipairs(self.parent.controls) do
			self.parent.controls[k] = nil;
		end
		for k in ipairs(layouts[index].layout) do
			self.parent:DefaultControl(k, layouts[index]);
		end
		self.parent:UpdateControls();
		self.parent:SaveControls();
		self.parent:SetupControlOptions(layouts[index]);
		self.parent:Redraw();
		self.parent.window:SetLayout(layouts[index]);
	end, nil);
	self.layoutDD:SetSize(self.width, self.size);

	local setting;
	self.top = { };
	self.logDebuffs = DropDownMenu(self, 3, { "No Debuffs", "No Icon Debuffs", "Unknown Debuffs", "All Debuffs" }, nil, nil);
	self.logDebuffs:SetSize(self.width * 1.5, self.size);
	self.logBuffs = DropDownMenu(self, 1, { "No Buffs", "No Icon Buffs", "Unknown Buffs", "All Buffs" }, nil, nil);
	self.logBuffs:SetSize(self.width * 1.5, self.size);
	table.insert(self.top, self.logDebuffs);
	table.insert(self.top, self.logBuffs);
	setting = DropDownMenu(self, 1, { "Existent", "Missing" }, function(self, index)
		self.parent.controls[self.parent.index].missing = (index == 2);
		self.parent:Changed();
	end, function(index) return index and 2 or 1; end);
	setting:SetSize(self.width, self.size);
	setting.setting = "missing";
	table.insert(self.top, setting);
	local nonnegint = function(x)
		x = tonumber(x);
		if x < 0 or math.floor(x) ~= x then
			x = nil;
		end
		return x;
	end;
	table.insert(self.top, PartyFrameOptionsField(self, "threat", "seconds", 30, 65, self.size, nonnegint, tostring));
	table.insert(self.top, PartyFrameOptionsField(self, "fill", "Power", 60, 50, self.size, nonnegint, tostring));
	table.insert(self.top, PartyFrameOptionsField(self, "effectDuration", "seconds", 30, 65, self.size, nonnegint, tostring));

	self.effects = { };
	self.groups = { };

	self.bottom1 = { };
	table.insert(self.bottom1, PartyFrameOptionsButton(self, "Clear", function(self, args)
		if self.parent.index == 0 then
			self.parent.controls[self.parent.index] = { };
			self.parent.logSaved = { };
			self.parent:SaveLogSaved();
			self.parent:Redraw();
		elseif self.parent.controls[self.parent.index].color ~= nil then
			self.parent.controls[self.parent.index].color = { };
			self.parent:Changed();
		end
	end));
	table.insert(self.bottom1, PartyFrameOptionsButton(self, "Default", function(self, agrs)
		self.parent:DefaultControl(self.parent.index, self.parent.window.layout);
		self.parent:Changed();
	end));

	self.setupName = Turbine.UI.Lotro.TextBox();
	self.setupName:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	self.setupName:SetWantsKeyEvents(true);
	self.setupName.FocusLost = function(self, args)
		for _, v in pairs(self.parent.buttons) do
			v:Colorize();
		end
	end;
	self.setupName.KeyDown = function(self, args)
		if self:HasFocus() and args.Action == 162 then
			self.parent:Focus();
		end
	end;
	self.setupName:SetMultiline(false);
	self.setupName:SetFont(Turbine.UI.Lotro.Font.Verdana16);
	self.setupName:SetParent(self);
	self.setupName.parent = self;
	self.setupMenu = Turbine.UI.ContextMenu();
	self.setupDropDown = Turbine.UI.Button();
	self.setupDropDown:SetBackground("moebius92/PartyFrame/res/dropdown.tga");
	self.setupDropDown:SetSize(self.size, self.size);
	self.setupDropDown:SetParent(self);
	self.setupDropDown.parent = self;
	self.setupDropDown.MouseClick = function(self, args)
		local items = self.parent.setupMenu:GetItems();
		local f = function(self, args) self.parent.setupName:SetText(self:GetText()); self.parent.setupName:FocusLost(); end;
		local item;
		local t = { };

		items:Clear();
		if self.parent.setupSaved[self.parent.window.layout.name] ~= nil then
			for k in pairs(self.parent.setupSaved[self.parent.window.layout.name]) do
				table.insert(t, k);
			end
			table.sort(t);
			if #t > 0 then
				for _, v in ipairs(t) do
					item = Turbine.UI.MenuItem(v);
					item.parent = self.parent;
					item.Click = f;
					items:Add(item);
				end
				self.parent.setupMenu:ShowMenuAt(self.parent.setupName:PointToScreen(0, self.parent.setupName:GetHeight()));
			end
		end
	end;

	self.bottom2 = { };
	table.insert(self.bottom2, PartyFrameOptionsButton(self, "Save", function(self, args)
		local name = self.parent.setupName:GetText();
		local lname = self.parent.window.layout.name;

		if self.parent.index ~= 0 and name ~= nil and name ~= "" then
			if self.parent.setupSaved[lname] == nil then
				self.parent.setupSaved[lname] = { };
			end
			if self.parent.setupSaved[lname][name] == nil then
				self.parent.setupSaved[lname][name] = { };
			end
			self.parent.setupSaved[lname][name][self.parent.index] = tabledeepcopy(self.parent.controls[self.parent.index]);
			self.parent.buttons[self.parent.index]:Colorize();
			self.parent:SaveControlsSaved();
		end
	end));
	table.insert(self.bottom2, PartyFrameOptionsButton(self, "Load", function(self, args)
		self.parent:LoadSavedControls(self.parent.setupName:GetText());
	end));
	table.insert(self.bottom2, PartyFrameOptionsButton(self, "Delete", function(self, args)
		local name = self.parent.setupName:GetText();
		local lname = self.parent.window.layout.name;

		if self.parent.setupSaved[lname] ~= nil and
		   self.parent.setupSaved[lname][name] ~= nil then
			self.parent.setupSaved[lname][name][self.parent.index] = nil;
			if #self.parent.setupSaved[lname][name] == 0 then
				self.parent.setupSaved[lname][name] = nil;
				if next(self.parent.setupSaved[lname]) == nil then
					self.parent.setupSaved[lname] = nil;
				end
			end
			self.parent.buttons[self.parent.index]:Colorize();
			self.parent:SaveControlsSaved();
		end
	end));

	self.buttons = { };
	self:SetupControlOptions(self.window.layout);
end

function PartyFrameOptions:LoadControls()
	local t = Turbine.PluginData.Load(Turbine.DataScope.Character, "PartyFrame_Controls");
	if type(t) == "string" then
		t = loadstring("return " .. t .. ";")();
	end

	if t == nil or t.version ~= self.window.version or t.layout ~= self.window.layout.name then
		t = { };
	end

	self.controls = { };
	for k, v in ipairs(self.window.layout.layout) do
		if t[k] ~= nil then
			self.controls[k] = t[k];
		else
			self:DefaultControl(k, self.window.layout);
		end
	end
end

function processDefault(k, v)
	if k == "color" then
		local t = { };

		for k1, v1 in ipairs(v) do
			t[k1] = { };
			for k2, v2 in pairs(v1) do
				t[k1][k2] = { };
				if k2 == "color" then
					t[k1].color.H = v2.H;
					t[k1].color.S = v2.S;
					t[k1].color.V = v2.V;

					if v2.R ~= nil then
						local r = v2.R;
						local g = v2.G;
						local b = v2.B;

						local max = math.max(r, g, b);
						local min = math.min(r, g, b);

						if max == 0.0 then
							t[k1].color.H = 0.0;
							t[k1].color.S = 0.0;
							t[k1].color.V = 0.0;
						else
							r = (r - min) / (max - min);
							g = (g - min) / (max - min);
							b = (b - min) / (max - min);

							if r == 1.0 then
								t[k1].color.H = (0.0 + g - b) % 6;
							elseif g == 1.0 then
								t[k1].color.H = (2.0 + b - r) % 6;
							elseif b == 1.0 then
								t[k1].color.H = (4.0 + r - g) % 6;
							else
								t[k1].color.H = 0.0;
							end
							t[k1].color.S = (max - min) / max;
							t[k1].color.V = max;
						end
					end
				else
					t[k1][k2].name = v2;
					if buffIcons[v2] ~= nil then
						t[k1][k2].icon = buffIcons[v2];
						t[k1][k2].debuff = false;
					elseif debuffIcons[v2] ~= nil then
						t[k1][k2].icon = debuffIcons[v2];
						t[k1][k2].debuff = true;
					else
						t[k1][k2].icon = nil;
						t[k1][k2].debuff = true;
					end
				end
			end
		end

		return t;
	else
		return v;
	end
end

function PartyFrameOptions:DefaultControl(i, layout)
	self.controls[i] = { };

	if i == 0 then
		self.controls[0] = tabledeepcopy(self.logSaved);
	else
		for k, v in pairs(layout.layout[i].default) do
			self.controls[i][k] = processDefault(k, v);
		end

		if layout.class[self.window.class] ~= nil and layout.layout[i].name ~= nil and
		   layout.class[self.window.class][layout.layout[i].name] ~= nil then
			for k, v in pairs(layout.class[self.window.class][layout.layout[i].name]) do
				self.controls[i][k] = processDefault(k, v);
			end
		end

		if self.controls[i].missing == nil then
			self.controls[i].missing = false;
		end
	end
end

function PartyFrameOptions:SaveControls()
	local t = { };
 
	for k, v in pairs(self.controls) do
		t[k] = v;
	end
	t.version = self.window.version;
	t.layout = self.window.layout.name;
	Turbine.PluginData.Save(Turbine.DataScope.Character, "PartyFrame_Controls", serialize(t));
end

function PartyFrameOptions:LoadLogSaved()
	self.logSaved = Turbine.PluginData.Load(Turbine.DataScope.Account, "PartyFrame_LogSaved");
	if type(self.logSaved) == "string" then
		self.logSaved = loadstring("return " .. self.logSaved .. ";")();
	end
	if self.logSaved == nil or self.logSaved.version ~= self.window.version then
		self.logSaved = { };
	end
	self.logSaved.version = nil;
end

function PartyFrameOptions:SaveLogSaved()
	self.logSaved.version = self.window.version;
	Turbine.PluginData.Save(Turbine.DataScope.Account, "PartyFrame_LogSaved", serialize(self.logSaved));
	self.logSaved.version = nil;
end

function PartyFrameOptions:LoadSavedControls(name)
	local lname = self.window.layout.name;

	if self.setupSaved[lname] ~= nil and self.setupSaved[lname][name] ~= nil then
		for k, v in pairs(self.setupSaved[lname][name]) do
			self.controls[k] = tabledeepcopy(v);
		end
		self:Changed();

		self.setupName:SetText(name);
		for _, v in pairs(self.buttons) do
			v:Colorize();
		end

		return true;
	end

	return false;
end

function PartyFrameOptions:LoadControlsSaved()
	self.setupSaved = Turbine.PluginData.Load(Turbine.DataScope.Account, "PartyFrame_SetupSaved");
	if type(self.setupSaved) == "string" then
		self.setupSaved = loadstring("return " .. self.setupSaved .. ";")();
	end
	if self.setupSaved == nil or self.setupSaved.version ~= self.setupSaved.version then
		self.setupSaved = { };
	end
	for k in pairs(self.setupSaved) do
		local found = false

		for _, v in pairs(layouts) do
			if v.name == k then
				found = true;
				break;
			end
		end

		if not found then
			self.setupSaved[k] = nil;
		end
	end
	self.setupSaved.version = nil;
end

function PartyFrameOptions:SaveControlsSaved()
	self.setupSaved.version = self.window.version;
	Turbine.PluginData.Save(Turbine.DataScope.Account, "PartyFrame_SetupSaved", serialize(self.setupSaved));
	self.setupSaved.version = nil;
end

function PartyFrameOptions:UpdateControls()
	self.window.effectControl = { };

	for k, v in ipairs(self.controls) do
		if v.color ~= nil then
			for k1, v1 in ipairs(v.color) do
				for k2, v2 in ipairs(v1) do
					local id;

					if string.sub(v2.name, 1, 1) == "[" then
						id = v2.name;
					elseif v2.icon ~= nil then
						id = (v2.debuff and "D" or "B") .. "/" .. v2.icon .. "/" .. v2.name;
					else
						id = (v2.debuff and "D" or "B") .. "/" .. v2.name;
					end

					if self.window.effectControl[id] == nil then
						self.window.effectControl[id] = { };
					end
					table.insert(self.window.effectControl[id], { control = k, group = k1 });
				end
			end
		end
	end

	self.window:UpdateControls();
end

function PartyFrameOptions:Select(i)
	if self.index ~= i then
		self.index = i;

		if self.index == 0 then
			for _, v in ipairs(self.top) do
				v:SetVisible(false);
			end

			self.logDebuffs:SetVisible(true);
			self.logBuffs:SetVisible(true);
		else
			for _, v in ipairs(self.top) do
				if v.setting ~= nil and 
				   (self.window.layout.layout[i].default[v.setting] ~= nil or
				    (self.window.layout.layout[i].default.class ~= nil and
					 self.window.layout.layout[i].default.class[self.window.class] ~= nil and
					 self.window.layout.layout[i].default.class[self.window.class][v.setting] ~= nil)) then
					v:SetVisible(true);
					v:SetValue(self.controls[i][v.setting]);
				else
					v:SetVisible(false);
				end
			end
		end

		self:Redraw();
	end
end

function PartyFrameOptions:SetupControlOptions(layout)
	for _, v in pairs(self.buttons) do
		v:SetParent(nil);
	end

	self.buttons = { };
	for k, v in pairs(layout.layout) do
		self.buttons[k] = PartyFrameOptionsControl(self, k, v.name);
	end
	table.insert(self.buttons, PartyFrameOptionsControl(self, 0, "Log"));
	self:Select(0);
end

function PartyFrameOptions:Layout(controls, y, h, m)
	local w = self:GetWidth();
	local t = { };
	local q = { };
	local c, i, l, p, x;

	for k, v in ipairs(controls) do
		t[k] = v:GetWidth();
	end

	i = 1;
	c = 0;
	p = { };
	while t[i] ~= nil do
		l = w - 4 * m;
		l = l - t[i];
		repeat
			table.insert(q, { i = i + 1, c = c + (l * l), p = { e = i, p = p } });
			i = i + 1;
			if t[i] ~= nil then
				l = l - (m + t[i]);
			else
				break;
			end
		until l <= 0;
		table.sort(q, function(a, b) return a.c < b.c; end);
		i = table.remove(q, 1);
		c = i.c;
		p = i.p;
		i = i.i;
	end

	q = { };
	while p.p ~= nil do
		table.insert(q, 1, p.e);
		p = p.p;
	end

	c = 1;
	for _, v in ipairs(q) do
		x = w;
		for i = c, v, 1 do
			x = x - t[i];
		end
		x = (x - (v - c) * m) / 2;
		for i = c, v, 1 do
			controls[i]:SetPosition(x, y);
			x = x + t[i] + m;
		end

		y = y + h + m;
		c = v + 1;
	end
	return y;
end

function PartyFrameOptions:RemoveEffect(g, i)
	if self.index == 0 then
		return table.remove(self.controls[self.index], i);
	else
		local t = self.controls[self.index].color[g][i];

		table.remove(self.controls[self.index].color[g], i);
		if #self.controls[self.index].color[g] == 0 then
			table.remove(self.controls[self.index].color, g);
		end
		return t;
	end
end

function PartyFrameOptions:SetSetting(setting, value)
	self.controls[self.index][setting] = value;
	self:Changed();
end

function PartyFrameOptions:Changed()
	self:UpdateControls();
	self:SaveControls();
	self:Redraw();
end

function PartyFrameOptions:Redraw()
	for _, v in pairs(self.buttons) do
		v:Colorize();
	end
	self.w = nil;
	self:SizeChanged();
end

function PartyFrameOptions:SizeChanged()
	local w, h = self:GetSize();
	local x;
	local y = 0;
	local t = { };

	if self.w ~= w then
		y = y + 10;

		x = (w - self.layoutDD:GetWidth()) / 2;
		self.layoutDD:SetPosition(x, y);
		y = y + self.layoutDD:GetHeight();
		y = y + 10;

		y = self:Layout(self.buttons, y, self.size, 5) + 5;

		for _, v in ipairs(self.top) do
			if v:IsVisible() then
				table.insert(t, v);
			end
		end
		y = self:Layout(t, y, self.size, 5) + 15;

		for k, v in ipairs(self.effects) do
			v:SetVisible(false);
		end
		for k, v in ipairs(self.groups) do
			v:SetVisible(false);
		end
		if self.index == 0 then
			for k, v in ipairs(self.controls[0]) do
				if self.effects[k] == nil then
					self.effects[k] = PartyFrameOptionsEffect(self, v.icon, v.debuff, v.name, true);
				else
					self.effects[k]:SetIcon(v.icon);
					self.effects[k]:SetDebuff(v.debuff);
					self.effects[k]:SetName(v.name, true);
				end
				self.effects[k]:SetIndex(nil, k);
				self.effects[k]:SetSize(w - (6 + self.size), self.size + 2);
				self.effects[k]:SetPosition(4 + self.size, y);
				self.effects[k]:SetVisible(true);
				y = y + self.size + 2;
			end
		elseif self.controls[self.index].color ~= nil then
			local i = 1;

			for k, v in ipairs(self.controls[self.index].color) do
				if self.groups[k] == nil then
					self.groups[k] = PartyFrameOptionsGroup(self, k);
				end
				self.groups[k]:SetPosition(4, y + 1);
				self.groups[k]:Setup(#v);

				for k1, v1 in ipairs(v) do
					if self.effects[i] == nil then
						self.effects[i] = PartyFrameOptionsEffect(self, v1.icon, v1.debuff, v1.name, false);
					else
						self.effects[i]:SetIcon(v1.icon);
						self.effects[i]:SetDebuff(v1.debuff);
						self.effects[i]:SetName(v1.name, false);
					end
					self.effects[i]:SetIndex(k, k1);
					self.effects[i]:SetSize(w - (6 + self.size), self.size + 2);
					self.effects[i]:SetPosition(4 + self.size, y);
					self.effects[i]:SetVisible(true);
					i = i + 1;
					y = y + self.size + 2;
				end
			end

			if self.effects[i] == nil then
				self.effects[i] = PartyFrameOptionsEffect(self, nil, false, nil, false);
			else
				self.effects[i]:SetIcon(nil);
				self.effects[i]:SetDebuff(false);
				self.effects[i]:SetName(nil, false);
			end
			self.effects[i]:SetIndex(nil, nil);
			self.effects[i]:SetSize(w - (6 + self.size), self.size + 2);
			self.effects[i]:SetPosition(4 + self.size, y);
			self.effects[i]:SetVisible(true);
			i = i + 1;
			y = y + self.size + 2;
		end
		y = y + 10;

		y = self:Layout(self.bottom1, y, self.size, 5);

		local l = math.min(w - 4, self.width * 3.5);
		x = (w - l) / 2;
		self.setupName:SetSize(l - self.size, self.size);
		self.setupName:SetPosition(x, y);
		x = x + (l - self.size);
		self.setupDropDown:SetPosition(x, y);
		y = y + (self.size + 5);

		y = self:Layout(self.bottom2, y, self.size, 5) + 5;

		self:SetHeight(y);

		self.w = w;
	end
end