PartyFrameOptionsBrightness = class(Turbine.UI.Window);

function PartyFrameOptionsBrightness:Constructor(parent)
	Turbine.UI.Window.Constructor(self);

	self.parent = parent;
	self:SetSize(120, 35);
	self:SetMouseVisible(true);
	self:SetBackground("moebius92/PartyFrame/res/brightness.tga");
end

function PartyFrameOptionsBrightness:CalculateBrightness(x, y)
	if x >= 10 and x <= 110 and y >= 10 and y <= 25 then
		return (x - 10) / 100;
	end
	return nil;
end

function PartyFrameOptionsBrightness:MouseMove(args)
	local target = self.parent.target;
	target.parent.controls[target.parent.index].color[target.index].color.V = self:CalculateBrightness(args.X, args.Y);
	target:Colorize();
end

function PartyFrameOptionsBrightness:MouseLeave(args)
	local target = self.parent.target;
	target.parent.controls[target.parent.index].color[target.index].color.V = self.V;
	target:Colorize();
end

function PartyFrameOptionsBrightness:FocusGained(args)
	local target = self.parent.target;
	self.V = target.parent.controls[target.parent.index].color[target.index].color.V;
end

function PartyFrameOptionsBrightness:FocusLost(args)
	self:SetVisible(false);
end

function PartyFrameOptionsBrightness:MouseClick(args)
	if args.Button == Turbine.UI.MouseButton.Left then
		local target = self.parent.target;
		self.V = self:CalculateBrightness(args.X, args.Y);
		self:SetVisible(false);
		target.parent:Changed();
	end
end

PartyFrameOptionsColor = class(Turbine.UI.Window);

function PartyFrameOptionsColor:Constructor(parent)
	Turbine.UI.Window.Constructor(self);

	self.parent = parent;
	self:SetSize(95, 95);
	self:SetMouseVisible(true);
	self:SetBackground("moebius92/PartyFrame/res/color.tga");
end

function PartyFrameOptionsColor:CalculateColor(x, y)
	local dx = x - 47;
	local dy = y - 47;
	local s = math.sqrt(dx * dx + dy * dy);

	if s <= 39 then
		local c = { };
		local x;

		s = math.min(1.0, s / 37);
		if s == 0 then
			h = 0.0;
		else
			h = ((math.atan2(dx / s, dy / s) * 180 / math.pi + 90) % 360) / 60;
		end
		return h, s;
	end
	return nil, nil;
end

function PartyFrameOptionsColor:MouseMove(args)
	local target = self.parent.target;
	local h, s = self:CalculateColor(args.X, args.Y);

	target.parent.controls[target.parent.index].color[target.index].color.H = h;
	target.parent.controls[target.parent.index].color[target.index].color.S = s;
	target:Colorize();
end

function PartyFrameOptionsColor:MouseLeave(args)
	local target = self.parent.target;
	target.parent.controls[target.parent.index].color[target.index].color.H = self.H;
	target.parent.controls[target.parent.index].color[target.index].color.S = self.S;
	target:Colorize();
end

function PartyFrameOptionsColor:FocusGained(args)
	local target = self.parent.target;
	self.H = target.parent.controls[target.parent.index].color[target.index].color.H;
	self.S = target.parent.controls[target.parent.index].color[target.index].color.S;
end

function PartyFrameOptionsColor:FocusLost(args)
	self:SetVisible(false);
end

function PartyFrameOptionsColor:MouseClick(args)
	if args.Button == Turbine.UI.MouseButton.Left then
		local target = self.parent.target;
		local h, s = self:CalculateColor(args.X, args.Y);

		self.H = h;
		self.S = s;
		self:SetVisible(false);
		target.parent:Changed();
	end
end

PartyFrameOptionsGroup = class(Turbine.UI.Control);

function PartyFrameOptionsGroup:Constructor(parent, index)
	Turbine.UI.Control.Constructor(self);

	self.parent = parent;
	self.index = index;
	self.dragging = false;

	self:SetParent(parent);

	self.u = Turbine.UI.Control();
	self.u:SetParent(self);
	self.u:SetPosition(0, 0);
	self.u:SetSize(parent.size, parent.size);
	self.u:SetBackground("moebius92/PartyFrame/res/inv-ul.tga");
	self.u:SetBlendMode(Turbine.UI.BlendMode.Overlay);
	self.u:SetMouseVisible(false);
	self.d = Turbine.UI.Control();
	self.d:SetParent(self);
	self.d:SetSize(parent.size, parent.size);
	self.d:SetBackground("moebius92/PartyFrame/res/inv-ll.tga");
	self.d:SetBlendMode(Turbine.UI.BlendMode.Overlay);
	self.d:SetMouseVisible(false);
	self:Colorize();
end

function PartyFrameOptionsGroup:Colorize()
	local c = self.parent.controls[self.parent.index].color[self.index].color;

	if c.H ~= nil then
		local H = (c.H + 1) % 6;
		local rgb = { };

		rgb[0] = 1.0 - c.S;
		rgb[1] = 1.0 - c.S
		rgb[2] = 1.0 - c.S
		x = math.floor(H / 2);
		rgb[x] = rgb[x] + c.S;
		H = H % 2;
		if H < 1 then
			x = (x - 1) % 3;
			rgb[x] = rgb[x] + c.S * (1.0 - H);
		else
			x = (x + 1) % 3;
			rgb[x] = rgb[x] + c.S * (H - 1.0);
		end
		if c.V ~= nil then
			self:SetBackColor(Turbine.UI.Color(1.0, rgb[0] * c.V, rgb[1] * c.V, rgb[2] * c.V));
		else
			self:SetBackColor(Turbine.UI.Color(1.0, rgb[0], rgb[1], rgb[2]));
		end
	elseif c.V ~= nil then
		self:SetBackColor(Turbine.UI.Color(1.0, c.V, c.V, c.V));
	else
		self:SetBackColor(Turbine.UI.Color(1.0, 1.0, 1.0, 1.0));
	end
end

function PartyFrameOptionsGroup:Setup(l)
	if l > 0 then
		self:SetSize(self.parent.size, (self.parent.size + 2) * l - 2);

		if l == 1 then
			self.d:SetVisible(false);
		else
			self.d:SetPosition(0, (self.parent.size + 2) * (l - 1));
			self.d:SetVisible(true);
		end

		self:SetVisible(true);
	else
		self:SetVisible(false);
	end
	self:Colorize();
end

function PartyFrameOptionsGroup:MouseClick(args)
	if args.Button == Turbine.UI.MouseButton.Right then
		PartyFrameOptionsGroup.target = self;
		self.options.Color:SetChecked(self.parent.controls[self.parent.index].color[self.index].color.H ~= nil);
		self.options.Brightness:SetChecked(self.parent.controls[self.parent.index].color[self.index].color.V ~= nil);
		self.menu:ShowMenu();
	end
end

function PartyFrameOptionsGroup:MouseDown(args)
	if args.Button == Turbine.UI.MouseButton.Left then
		self.dragging = true;
		self.safeY = nil;
	end
end

function PartyFrameOptionsGroup:MouseMove(args)
	if self.dragging then
		if not IsSafeY(self, args.Y) then
			return;
		end

		local d = 0;

		if args.Y < 0 then
			d = -1;
		elseif args.Y > self:GetHeight() then
			d = 1;
		end

		if d ~= 0 then
			local i = self.index + d;

			if self.parent.groups[i] ~= nil then
				local x = self.parent.controls[self.parent.index].color[self.index];

				self.parent.controls[self.parent.index].color[self.index] = self.parent.controls[self.parent.index].color[i];
				self.parent.controls[self.parent.index].color[i] = x;
				self.parent.groups[self.index] = self.parent.groups[i];
				self.parent.groups[self.index].index = self.index;
				self.parent.groups[i] = self;
				self.index = i;
				self.parent:Changed();
				self.safeY = -d * math.huge;
			end
		end
	end
end

function PartyFrameOptionsGroup:MouseUp(args)
	if args.Button == Turbine.UI.MouseButton.Left then
		self.dragging = false;
	end
end

PartyFrameOptionsGroup.color = PartyFrameOptionsColor(PartyFrameOptionsGroup);
PartyFrameOptionsGroup.brightness = PartyFrameOptionsBrightness(PartyFrameOptionsGroup);
PartyFrameOptionsGroup.options = { };
PartyFrameOptionsGroup.menu = Turbine.UI.ContextMenu();
local items = PartyFrameOptionsGroup.menu:GetItems();
PartyFrameOptionsGroup.options.Color = Turbine.UI.MenuItem("Color");
PartyFrameOptionsGroup.options.Color.Click = function(self, args)
	PartyFrameOptionsGroup.color:SetPosition(Turbine.UI.Display.GetMousePosition());
	PartyFrameOptionsGroup.color:SetVisible(true);
	PartyFrameOptionsGroup.color:Activate();
	PartyFrameOptionsGroup.color:Focus();
end;
items:Add(PartyFrameOptionsGroup.options.Color);
PartyFrameOptionsGroup.options.Brightness = Turbine.UI.MenuItem("Brightness");
PartyFrameOptionsGroup.options.Brightness.Click = function(self, args)
	PartyFrameOptionsGroup.brightness:SetPosition(Turbine.UI.Display.GetMousePosition());
	PartyFrameOptionsGroup.brightness:SetVisible(true);
	PartyFrameOptionsGroup.brightness:Activate();
	PartyFrameOptionsGroup.brightness:Focus();
end;
items:Add(PartyFrameOptionsGroup.options.Brightness);
